import {
  schemeTableau10,
  scaleOrdinal,
  area,
  pie,
  arc,
  treemap,
  hierarchy,
} from 'd3';
import { select } from 'd3-selection';
export const treeMap = (
  selection,
  { data, height, width }
) => {
  // Set up the dimensions and margins for the SVG element
  const margin = {
    top: 50,
    right: 10,
    bottom: 10,
    left: 10,
  };

  const svg = selection
    .append('svg')
    .attr('width', width)
    .attr('height', height);

  svg
    .append('text')
    .attr('x', width / 2)
    .attr('y', margin.top / 1.5)
    .attr('text-anchor', 'middle')
    .style('font-size', '24px')
    .style('font-weight', 'bold')
    .style('font-family', 'sans-serif')
    .text('Artist Nationality');

  const colorScale = scaleOrdinal(
    schemeTableau10
  );

  const treemapLayout = treemap()
    .size([width, height - margin.top])
    .padding(4)
    .round(true);

  // Create a hierarchy from the data
  const hierarchyData = {
    name: 'Artists',
    children: data.map((d) => {
      return {
        name: d.nationality,
        value: d.count,
      };
    }),
  };

  // Create the root node for the treemap
  const rootNode = hierarchy(hierarchyData)
    .sum((d) => d.value)
    .sort((a, b) => b.value - a.value);

  // Calculate the treemap layout
  treemapLayout(rootNode);

  // Add the cells to the SVG
  const cells = svg
    .selectAll('rect')
    .data(rootNode.leaves())
    .enter()
    .append('rect')
    .attr('x', (d) => d.x0)
    .attr('y', (d) => d.y0 + margin.top)
    .attr('width', (d) => d.x1 - d.x0)
    .attr('height', (d) => d.y1 - d.y0)
    .attr('stroke', '#fff')
    .attr('fill', (d) => colorScale(d.data.name))
    .on('mouseover', function () {
      const tooltip = select('#tooltip');
      const data = select(this).datum().data;
      tooltip.html(`${data.name}: ${data.value}`);
      tooltip.style('visibility', 'visible');
    })

    .on('mousemove', function () {
      const tooltip = select('#tooltip');
      const tooltipWidth = tooltip.node()
        .offsetWidth;
      const tooltipHeight = tooltip.node()
        .offsetHeight;
      const cursorX = event.pageX;
      const cursorY = event.pageY;

      // Check if the tooltip is too close to the right edge
      if (
        window.innerWidth - cursorX <
        tooltipWidth + 20
      ) {
        tooltip
          .style(
            'left',
            cursorX - tooltipWidth - 20 + 'px'
          )
          .style('top', cursorY + 10 + 'px');
      } else {
        tooltip
          .style('left', cursorX + 10 + 'px')
          .style('top', cursorY + 10 + 'px');
      }
    })
    .on('mouseout', function () {
      const tooltip = select('#tooltip');
      tooltip.style('visibility', 'hidden');
    });

  svg
    .selectAll('text')
    .data(rootNode.descendants())
    .enter()
    .append('text')
    .attr('x', (d) => d.x0 + (d.x1 - d.x0) / 2)
    .attr(
      'y',
      (d) => d.y0 + margin.top + (d.y1 - d.y0) / 2
    )
    .attr('text-anchor', 'middle')
    .style('font-size', '18px')
    .style('font-weight', 'bold')
    .style('fill', '#fff')
    .style('font-family', 'sans-serif')
    .text((d) => {
      if (
        d.parent &&
        d.x1 - d.x0 > 70 &&
        d.y1 - d.y0 > 50
      ) {
        return d.data.name;
      } else if (!d.parent) {
        return 'Artists'; // set the label for the parent node here
      } else {
        return '';
      }
    });

  // Add a tooltip element to the page
  const tooltip = select('body')
    .append('div')
    .attr('id', 'tooltip')
    .style('position', 'absolute')
    .style('background-color', '#fff')
    .style('border', '1px solid #ddd')
    .style('padding', '5px')
    .style('visibility', 'hidden');
};
